#!/usr/bin/python

# This file is part of the PiXtend(R) Project.
#
# For more information about PiXtend(R) and this program,
# see <https://www.pixtend.de> or <https://www.pixtend.com>
#
# Copyright (C) 2019 Robin Turner
# Qube Solutions GmbH, Arbachtalstr. 6
# 72800 Eningen, Germany
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Made for Python 2 and pyModBus
# For this example to work a wire must be connected to AO0 and AI0

import RPi.GPIO as GPIO
from pymodbus.client.sync import ModbusSerialClient as ModbusClient
import time

# Activate RS485: Only needed for PiXtend V2 -L-
GPIO.setwarnings(False)
GPIO.setmode(GPIO.BCM)
GPIO.setup(18, GPIO.OUT)
GPIO.output(18, GPIO.HIGH)

# Serial configuration for RPi miniUART ttyS0, no parity
#client= ModbusClient(method = "rtu", port="/dev/ttyS0",stopbits = 1, bytesize = 8, parity = 'N', baudrate= 19200)

# Serial configuration for RPi full UART ttyAMA0
client= ModbusClient(method = "rtu", port="/dev/ttyAMA0",stopbits = 1, bytesize = 8, parity = 'E', baudrate= 19200)

# Serial configuration for USB-to-RS485 dongle
#client= ModbusClient(method = "rtu", port="/dev/ttyUSB0",stopbits = 1, bytesize = 8, parity = 'E', baudrate= 19200)

connection = client.connect()

while True:
    try:
        print("Write AO0 to 2048, about 5 volts")
        # Write register 0 and set AO0 to 5 volts
        result= client.write_register(0x00, 2048, unit= 0x03)
        time.sleep(1)
        # Read the inputs reagister for AI0
        result= client.read_input_registers(0x00, 1, unit= 0x03)
        print("Read AI0: " + str(result.registers[0]))
        time.sleep(1)
        print("Write AO0 to 0")
        # Turn AO0 off, writing 0 to its register
        result= client.write_register(0x00,0, unit= 0x03)
        time.sleep(1)
        # Read AI0 again, should be 0 now as well
        result= client.read_input_registers(0x00, 1, unit= 0x03)
        print("Read AI0: " + str(result.registers[0]))
        time.sleep(1)        
    except KeyboardInterrupt:
        # Pressing Ctrl + C will end the program
        client.close()
        time.sleep(0.25)
        client = None
        break



