﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.IO.Ports;
using System.Threading;
using System.IO;
using PiXtend_eIO_Demo.Properties;

/*
# This file is part of the PiXtend(R) Project.
#
# For more information about PiXtend(R) and this program,
# see <https://www.pixtend.de> or <https://www.pixtend.com>
#
# Copyright (C) 2019 Robin Turner
# Qube Solutions GmbH, Arbachtalstr. 6
# 72800 Eningen, Germany
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/*
    ------------------------------------
    Digital One
    ------------------------------------
    Protocol length: 16
    - Set DO0 of Device 1 to High
      #001:003:01:001*
    - Set DO0 of Device 1 to Low
      #001:003:01:000*
    - Read DI1 of Device 1
      #001:001:02:002*

    ------------------------------------
    Analog One
    ------------------------------------
    Protocol length: 17
    - Set AO1 to max
    #003:006:01:4095*
    - Set AO1 to 0
    #003:006:01:0000*
 
 */

namespace PiXtend_eIO_Demo
{
    public enum DeviceType  {AnalogOne, DigitalOne, Unknown};
    public enum PinType     {AI, AO, DI, DO, S, Unknown};
    public enum CommandType {CommandReadDi=1, CommandReadDiAll, CommandWriteDo, CommandWriteDoAll, CommandReadAnalogIn, CommandWriteAnalogOut, CommandReadStatus, Unknown};

    public partial class Form1 : Form
    {
        public string DigitalOneName = "Digital One";
        public string AnalogOneName = "Analog One";

        private int AnalogOneCmdLength = 17;
        private int DigitalOneCmdLength = 16;
        private DeviceType CurrentDevice;
        private PinType CurrentPinType;

        // The main control for communicating through the RS-232/485 port
        private SerialPort comport = new SerialPort();
        int NumberOfDevices = 16;
        string ResponseData;
        bool isResponseOK;
        bool WaitingForResponse;
        int ResponseWaitTimeout;
        int CurrentEntry;

        private Settings settings = Settings.Default;

        public Form1()
        {
            InitializeComponent();

            cmbParity.Text = "Even";
            cmbDataBits.Text = "8";
            cmbStopBits.Text = "1";

            comport.DataReceived += new SerialDataReceivedEventHandler(port_DataReceived);

            // Load user settings
            settings.Reload();
        }

        /// <summary> Enable/disable controls based on the app's current state. </summary>
        private void EnableControls()
        {
            // Enable/disable controls based on whether the port is open or not
            gbPortSettings.Enabled = !comport.IsOpen;
            btnAutoSend.Enabled = comport.IsOpen;
            cmdAddDevice.Enabled = !comport.IsOpen;
            cmdRemoveDevice.Enabled = !comport.IsOpen;

            loadListToolStripMenuItem.Enabled = !comport.IsOpen;
            saveListToolStripMenuItem.Enabled = !comport.IsOpen;

            if (comport.IsOpen) btnOpenPort.Text = "&Close Port";
            else btnOpenPort.Text = "&Open Port";
        }

        private void cmdAddDevice_Click(object sender, EventArgs e)
        {
            contextMenuStrip1.Show(Cursor.Position);
        }

        private void cmdRemoveDevice_Click(object sender, EventArgs e)
        {
            if (lvPiXtendeIOBus.SelectedIndices.Count != 0)
            {
                int iTemp = lvPiXtendeIOBus.SelectedItems[0].Index;

                lvPiXtendeIOBus.SelectedItems[0].Remove();
                NumberOfDevices--;

                if (NumberOfDevices > 0)
                {
                    for (int i = 1; i < lvPiXtendeIOBus.Items.Count+1; i++)
                    {
                        lvPiXtendeIOBus.Items[i-1].Text = i.ToString();
                    }
                }

                if (iTemp != 0)
                {
                    lvPiXtendeIOBus.Items[iTemp - 1].Selected = true;
                }
                else
                {
                    if (lvPiXtendeIOBus.Items.Count >= 1)
                    {
                        lvPiXtendeIOBus.Items[0].Selected = true;
                    }
                }
            }

            if (lvPiXtendeIOBus.Items.Count == 0)
            {
                cmdRemoveDevice.Enabled = false;
                NumberOfDevices = 0;
            }
        }

        private void tsmDigitalOne_Click(object sender, EventArgs e)
        {
            int DevNumTemp = NumberOfDevices;
            ListViewItem lvi;

            DevNumTemp++;
            lvi = new ListViewItem(new string[] {DevNumTemp.ToString(), DigitalOneName, DevNumTemp.ToString(), "DI 0", "0", "-"} );
            lvPiXtendeIOBus.Items.Add(lvi);
            NumberOfDevices = DevNumTemp;

            if (lvPiXtendeIOBus.Items.Count > 0)
            {
                cmdRemoveDevice.Enabled = true;
            }
        }

        private void tsmAnalogOne_Click(object sender, EventArgs e)
        {
            int DevNumTemp = NumberOfDevices;
            ListViewItem lvi;

            DevNumTemp++;
            lvi = new ListViewItem(new string[] { DevNumTemp.ToString(), AnalogOneName, DevNumTemp.ToString(), "AI 0", "0", "-" });
            lvPiXtendeIOBus.Items.Add(lvi);
            NumberOfDevices = DevNumTemp;

            if (lvPiXtendeIOBus.Items.Count > 0)
            {
                cmdRemoveDevice.Enabled = true;
            }
        }

        private void lvPiXtendeIOBus_DoubleClick(object sender, EventArgs e)
        {
            if (lvPiXtendeIOBus.SelectedIndices.Count != 0)
            {
                using (var pdlg = new PropertiesDlg())
                {
                    pdlg.ListNumber = int.Parse(lvPiXtendeIOBus.SelectedItems[0].Text);
                    pdlg.DeviceID = int.Parse(lvPiXtendeIOBus.SelectedItems[0].SubItems[2].Text);
                    pdlg.DeviceType = lvPiXtendeIOBus.SelectedItems[0].SubItems[1].Text;
                    pdlg.DataType = lvPiXtendeIOBus.SelectedItems[0].SubItems[3].Text;
                    pdlg.Value = int.Parse(lvPiXtendeIOBus.SelectedItems[0].SubItems[4].Text);
                    pdlg.StartPosition = FormStartPosition.CenterParent;

                    var result = pdlg.ShowDialog(this);
                    if (result == DialogResult.OK)
                    {
                        string val1 = pdlg.DataType;
                        string val2 = pdlg.DeviceType;
                        int val3 = pdlg.DeviceID;
                        int val4 = pdlg.Value;
                        //Assign values back to listview
                        Console.WriteLine(val1);
                        Console.WriteLine(val2);
                        Console.WriteLine(val3.ToString());
                        lvPiXtendeIOBus.SelectedItems[0].SubItems[3].Text = val1;
                        lvPiXtendeIOBus.SelectedItems[0].SubItems[1].Text = val2;
                        lvPiXtendeIOBus.SelectedItems[0].SubItems[2].Text = val3.ToString();
                        lvPiXtendeIOBus.SelectedItems[0].SubItems[4].Text = val4.ToString();
                        lvPiXtendeIOBus.SelectedItems[0].SubItems[5].Text = "-";
                    }
                }
            }
        }

        private void tmrCheckComPorts_Tick(object sender, EventArgs e)
        {
            // checks to see if COM ports have been added or removed
            // since it is quite common now with USB-to-Serial adapters
            RefreshComPortList();
        }

        private void RefreshComPortList()
        {
            // Determain if the list of com port names has changed since last checked
            string selected = RefreshComPortList(cmbPortName.Items.Cast<string>(), cmbPortName.SelectedItem as string, comport.IsOpen);

            // If there was an update, then update the control showing the user the list of port names
            if (!String.IsNullOrEmpty(selected))
            {
                cmbPortName.Items.Clear();
                cmbPortName.Items.AddRange(OrderedPortNames());
                cmbPortName.SelectedItem = selected;
            }
        }

        private string[] OrderedPortNames()
        {
            // Just a placeholder for a successful parsing of a string to an integer
            int num;

            // Order the serial port names in numberic order (if possible)
            return SerialPort.GetPortNames().OrderBy(a => a.Length > 3 && int.TryParse(a.Substring(3), out num) ? num : 0).ToArray();
        }

        private string RefreshComPortList(IEnumerable<string> PreviousPortNames, string CurrentSelection, bool PortOpen)
        {
            // Create a new return report to populate
            string selected = null;

            // Retrieve the list of ports currently mounted by the operating system (sorted by name)
            string[] ports = SerialPort.GetPortNames();

            // First determain if there was a change (any additions or removals)
            bool updated = PreviousPortNames.Except(ports).Count() > 0 || ports.Except(PreviousPortNames).Count() > 0;

            // If there was a change, then select an appropriate default port
            if (updated)
            {
                // Use the correctly ordered set of port names
                ports = OrderedPortNames();

                // Find newest port if one or more were added
                string newest = SerialPort.GetPortNames().Except(PreviousPortNames).OrderBy(a => a).LastOrDefault();

                // If the port was already open... (see logic notes and reasoning in Notes.txt)
                if (PortOpen)
                {
                    if (ports.Contains(CurrentSelection)) selected = CurrentSelection;
                    else if (!String.IsNullOrEmpty(newest)) selected = newest;
                    else selected = ports.LastOrDefault();
                }
                else
                {
                    if (!String.IsNullOrEmpty(newest)) selected = newest;
                    else if (ports.Contains(CurrentSelection)) selected = CurrentSelection;
                    else selected = ports.LastOrDefault();
                }
            }

            // If there was a change to the port list, return the recommended default selection
            return selected;
        }

        private void btnOpenPort_Click(object sender, EventArgs e)
        {
            bool error = false;

            // If the port is open, close it.
            if (comport.IsOpen) comport.Close();
            else
            {
                // Set the port's settings
                comport.BaudRate = int.Parse(cmbBaudRate.Text);
                comport.DataBits = int.Parse(cmbDataBits.Text);
                comport.StopBits = (StopBits)Enum.Parse(typeof(StopBits), cmbStopBits.Text);
                comport.Parity = (Parity)Enum.Parse(typeof(Parity), cmbParity.Text);
                comport.PortName = cmbPortName.Text;

                try
                {
                    // Open the port
                    comport.Open();
                }
                catch (UnauthorizedAccessException) { error = true; }
                catch (IOException) { error = true; }
                catch (ArgumentException) { error = true; }

                if (error) MessageBox.Show(this, "Could not open the COM port.  Most likely it is already in use, has been removed, or is unavailable.", "COM Port Unavalible", MessageBoxButtons.OK, MessageBoxIcon.Stop);
            }

            // Change the state of the form's controls
            EnableControls();
        }

        private void port_DataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            // If the com port has been closed, do nothing
            if (!comport.IsOpen) return;
            isResponseOK = false;
            ResponseData = "";
            // This method will be called when there is data waiting in the port's buffer
            // Read all the data waiting in the buffer up to the * char
            string data = comport.ReadTo("*");
            data += "*";

            //Check which reply we are expecting
            if (data.Length == AnalogOneCmdLength && CurrentDevice == DeviceType.AnalogOne)
            {
                if (data[0] == '#' && data[4] == ':' && data[8] == ':' && data[11] == ':' && data[16] == '*')
                {
                    ResponseData = data;
                    isResponseOK = true;
                    WaitingForResponse = false;
                }
                else
                {
                    Console.WriteLine(data + " <-- Protocol Error!\n");
                }
            }
            else if (data.Length == DigitalOneCmdLength && CurrentDevice == DeviceType.DigitalOne)
            {
                if (data[0] == '#' && data[4] == ':' && data[8] == ':' && data[11] == ':' && data[15] == '*')
                {
                    ResponseData = data;
                    isResponseOK = true;
                    WaitingForResponse = false;
                }
                else
                {
                    Console.WriteLine(data + " <-- Protocol Error!\n");
                    WaitingForResponse = false;
                }
            }
            else
            {
                Console.WriteLine(data + " <-- Protocol Error!\n");
                WaitingForResponse = false;
            }
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            if (WaitingForResponse == false || ResponseWaitTimeout >= 8)
            {
                for (int i = 0; i < lvPiXtendeIOBus.Items.Count; i++)
                {
                    lvPiXtendeIOBus.Items[i].SubItems[5].Text = "-";
                }

                if (isResponseOK == true)
                {
                    int EntryToMod = CurrentEntry - 1;
                    if (EntryToMod < 0)
                    {
                        EntryToMod = lvPiXtendeIOBus.Items.Count - 1;
                    }

                    if (CurrentDevice == DeviceType.AnalogOne && CurrentPinType == PinType.AI && ResponseData.Length > 0)
                    {
                        lvPiXtendeIOBus.Items[EntryToMod].SubItems[4].Text = ResponseData.Substring(12, 4);
                    }
                    else if (CurrentDevice == DeviceType.DigitalOne && CurrentPinType == PinType.DI && ResponseData.Length > 0)
                    {
                        lvPiXtendeIOBus.Items[EntryToMod].SubItems[4].Text = ResponseData.Substring(12, 3);
                    }
                    else
                    {
                        ; //lvPiXtendeIOBus.Items[CurrentEntry].SubItems[4].Text = "?";
                    }
                }

                WaitingForResponse = false;
                isResponseOK = false;
                AutoSendData();
                ResponseWaitTimeout = 0;
            }
            else
            {
                ResponseWaitTimeout += 1;
            }
        }

        private void AutoSendData()
        {
            string sCmd = BuildCommand();
            // Send the chosen command out of the serial port
            comport.Write(sCmd);
            
            lvPiXtendeIOBus.Items[CurrentEntry].SubItems[5].Text = "<->";

            CurrentEntry += 1;
            if (CurrentEntry >= lvPiXtendeIOBus.Items.Count)
            {
                CurrentEntry = 0;
            }

            WaitingForResponse = true;
        }

        private DeviceType GetDeviceType (string DeviceName)
        {
            if (DeviceName == DigitalOneName)
            {
                return DeviceType.DigitalOne;
            }
            else if (DeviceName == AnalogOneName)
            {
                return DeviceType.AnalogOne;
            }
            else
            {
                return DeviceType.Unknown;
            }
        }

        private PinType GetPinType (string sDataType)
        {
            if (sDataType.Contains("AI") == true)
            {
                return PinType.AI;
            }
            else if (sDataType.Contains("AO") == true)
            {
                return PinType.AO;
            }
            else if (sDataType.Contains("DI") == true)
            {
                return PinType.DI;
            }
            else if (sDataType.Contains("DO") == true)
            {
                return PinType.DO;
            }
            else if (sDataType.Contains("S") == true)
            {
                return PinType.S;
            }
            else
            {
                return PinType.Unknown;
            }
        }

        private string GetPinNumber (string sDataType)
        {
            string sTemp;

            sTemp = "";
            sTemp = sDataType.Substring(sDataType.Length - 1);
            sTemp = sTemp.Trim();

            return sTemp;
        }

        private string BuildDeviceID(string sDeviceID)
        {
            if (sDeviceID.Length == 1)
            {
                return String.Concat("00", sDeviceID);
            }
            else if (sDeviceID.Length == 2)
            {
                return String.Concat("0", sDeviceID);
            }
            else if (sDeviceID.Length == 2)
            {
                return sDeviceID;
            }
            else
            {
                return "000";
            }

        }

        private string BuildOutputValue (string sValue, PinType pt)
        {
            if (pt == PinType.AO)
            {
                if (sValue.Length == 1)
                {
                    return string.Concat("000", sValue);
                }
                else if (sValue.Length == 2)
                {
                    return string.Concat("00", sValue);
                }
                else if (sValue.Length == 3)
                {
                    return string.Concat("0", sValue);
                }
                else if (sValue.Length == 4)
                {
                    return sValue;
                }
                else
                {
                    return sValue;
                }
            }
            else if (pt == PinType.DO)
            {
                if (sValue.Length == 1)
                {
                    return string.Concat("00", sValue);
                }
                else if (sValue.Length == 2)
                {
                    return string.Concat("0", sValue);
                }
                else if (sValue.Length == 3)
                {
                    return sValue;
                }
                else
                {
                    return sValue;
                }
            }
            else
            {
                if (pt == PinType.AI)
                {
                    return "0000";
                }
                else if (pt == PinType.DI)
                {
                    return "000";
                }
                else if (pt == PinType.S)
                {
                    if (CurrentDevice == DeviceType.AnalogOne)
                    {
                        return "0000";
                    }
                    else if (CurrentDevice == DeviceType.DigitalOne)
                    {
                        return "000";
                    }
                    else
                    {
                        return "-1";
                    }
                }
                else
                {
                    return "-1";
                }
            }
        }

        private string BuildCommand ()
        {
            //Target: #037:003:01:001* or #037:005:01:0001*
            //           Digital One         Analog One

            string sTempCmd;
            DeviceType dt;
            PinType pt;

            //Which device are we dealing with: Analog One or Digital One
            dt = GetDeviceType(lvPiXtendeIOBus.Items[CurrentEntry].SubItems[1].Text);
            CurrentDevice = dt;

            //Which pin / connection type do we have: Input, Output, Analog or Digital or Status
            pt = GetPinType(lvPiXtendeIOBus.Items[CurrentEntry].SubItems[3].Text);
            CurrentPinType = pt;

            //Protocol start
            sTempCmd = "#";
            //Device ID
            sTempCmd = string.Concat(sTempCmd, BuildDeviceID(lvPiXtendeIOBus.Items[CurrentEntry].SubItems[2].Text));
            //Devider
            sTempCmd = string.Concat(sTempCmd, ":");
            //Command
            if (pt == PinType.AI)
            {
                //Read Analog Input from Analog One module, Command number 5
                sTempCmd = string.Concat(sTempCmd, "005");
            }
            else if (pt == PinType.AO)
            {
                //Write Analog Output on Analog One module, Command number 6
                sTempCmd = string.Concat(sTempCmd, "006");
            }
            else if (pt == PinType.DI)
            {
                //Read Digital Input from Digital One module, Command number 1
                sTempCmd = string.Concat(sTempCmd, "001");
            }
            else if (pt == PinType.DO)
            {
                //Write Digital Output on Digital One module, Command number 3
                sTempCmd = string.Concat(sTempCmd, "003");
            }
            else if (pt == PinType.S)
            {
                //Read the Status byte from Analog One, Command number 200
                sTempCmd = string.Concat(sTempCmd, "200");
            }
            else
            {
                //Type not known
                sTempCmd = string.Concat(sTempCmd, "000");
            }
            //Devider
            sTempCmd = string.Concat(sTempCmd, ":");
            //Pin number
            sTempCmd = string.Concat(sTempCmd, "0");
            if (pt == PinType.S)
            {
                //For status concat 00 for first byte, low byte, just as an example
                sTempCmd = string.Concat(sTempCmd, "0");
            }
            else
            {
                //Pin numbers go from 0 to 7 max right now
                sTempCmd = string.Concat(sTempCmd, GetPinNumber(lvPiXtendeIOBus.Items[CurrentEntry].SubItems[3].Text));
            }
 
            //Devider
            sTempCmd = string.Concat(sTempCmd, ":");

            if (pt == PinType.S)
            {
                //For status concat 00 for first byte, low byte, just as an example
                sTempCmd = string.Concat(sTempCmd, BuildOutputValue("0", pt));
            }
            else
            {
                //Value for Output, for Inputs we use a 0
                sTempCmd = string.Concat(sTempCmd, BuildOutputValue(lvPiXtendeIOBus.Items[CurrentEntry].SubItems[4].Text, pt));
            }

            //Stop char
            sTempCmd = string.Concat(sTempCmd, "*");
            return sTempCmd;
        }

        private void btnAutoSend_Click(object sender, EventArgs e)
        {
            if (timer1.Enabled == false)
            {
                timer1.Enabled = true;
                btnAutoSend.Text = "Stop";
            }
            else
            {
                timer1.Enabled = false;
                btnAutoSend.Text = "Start";
                for (int i = 0; i < lvPiXtendeIOBus.Items.Count; i++)
                {
                    lvPiXtendeIOBus.Items[i].SubItems[5].Text = "-";
                }
            }
            
        }

        private void saveListToolStripMenuItem_Click(object sender, EventArgs e)
        {
            settings.ListItems = new System.Collections.Specialized.StringCollection();

            for (int i = 0; i < lvPiXtendeIOBus.Items.Count; i++)
            {
                string sTemp = "";
                sTemp = lvPiXtendeIOBus.Items[i].SubItems[1].Text;
                sTemp = string.Concat(sTemp, "|", lvPiXtendeIOBus.Items[i].SubItems[2].Text);
                sTemp = string.Concat(sTemp, "|", lvPiXtendeIOBus.Items[i].SubItems[3].Text);
                sTemp = string.Concat(sTemp, "|", lvPiXtendeIOBus.Items[i].SubItems[4].Text);
                settings.ListItems.Add(sTemp);
            }

            settings.FirstRun = true;
            settings.Save();

            MessageBox.Show("Current list successfully saved!", "List Saved", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        private void loadListToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (settings.FirstRun == true)
            {
                lvPiXtendeIOBus.Items.Clear();
                NumberOfDevices = 0;

                for (int i = 0; i < settings.ListItems.Count; i++)
                {
                    string[] sTemp;
                    sTemp = settings.ListItems[i].Split('|');
                    lvPiXtendeIOBus.Items.Add(new ListViewItem(new string[] {(i+1).ToString(), sTemp[0], sTemp[1], sTemp[2], sTemp[3], "-" }));
                }
                NumberOfDevices = lvPiXtendeIOBus.Items.Count;

                cmdAddDevice.Enabled = !comport.IsOpen;
                cmdRemoveDevice.Enabled = !comport.IsOpen;
            }
            else
            {
                MessageBox.Show("Sorry! Cannot load list, nothing was previously saved. Save a list first and try again to load it.","Error Loading List", MessageBoxButtons.OK,MessageBoxIcon.Error);
            }
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            timer1.Enabled = false;
            btnAutoSend.Text = "Start";
            for (int i = 0; i < lvPiXtendeIOBus.Items.Count; i++)
            {
                lvPiXtendeIOBus.Items[i].SubItems[5].Text = "-";
            }
            if (comport.IsOpen) comport.Close();
        }
    }
}
