package de.qubesolutions.pixtendeiodemorunninglights;

// This file is part of the PiXtend(R) Project.
//
// For more information about PiXtend(R) and this program,
// see <https://www.pixtend.de> or <https://www.pixtend.com>
//
// Copyright (C) 2019 Robin Turner
// Qube Solutions GmbH, Arbachtalstr. 6
// 72800 Eningen, Germany
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

import java.net.URL;
import java.util.ResourceBundle;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.Label;
import java.util.TimerTask;
import java.util.Timer;
import java.util.logging.Level;
import java.util.logging.Logger;

import jssc.SerialPort;
import jssc.SerialPortException;
import jssc.SerialPortTimeoutException;

public class FXMLController implements Initializable {
    private int Status;
    Timer timer;
    SerialPort serialPort;
    
    @FXML
    private Label label;
    
    @FXML
    private void handleButtonAction(ActionEvent event) {
        System.out.println("You clicked me!");
        if (Status == 0)
        {
            label.setText("Running!");
            Status = 1;  
            // initialization with port
            serialPort = new SerialPort("COM3");
            try {
               // open port for communication
               serialPort.openPort();
               // baundRate 19200, numberOfDataBits 8 , numberOfStopBits 1, parity EVEN
               serialPort.setParams(19200, 8, 1, 2);

                timer = new Timer();
                timer.scheduleAtFixedRate(new RunningLightsTask(serialPort), 100, 100);

            } catch (SerialPortException ex) {
               System.out.println(ex);
            }
        }
        else
        {
           label.setText("Stopped!");
           Status = 0;
           timer.cancel();
           timer = null;
           
           if (serialPort.isOpened() == true)
           {
                try {
                   // Close serial port
                   serialPort.purgePort(SerialPort.PURGE_RXCLEAR);
                   serialPort.purgePort(SerialPort.PURGE_TXCLEAR);
                   serialPort.closePort();
                   serialPort = null;
                } catch (SerialPortException ex) {
                   System.out.println(ex);
                }               
           }
        }
    }
    
    @Override
    public void initialize(URL url, ResourceBundle rb) {
        
    }    
}

class RunningLightsTask extends TimerTask
{
   private Boolean Running;
   private int iCurrentCommand;
   private String[] arsCommands =  {"#001:004:01:000*",
                                    "#001:004:01:001*",
                                    "#001:004:01:002*",
                                    "#001:004:01:004*",
                                    "#001:004:01:008*",
                                    "#001:004:01:016*",
                                    "#001:004:01:032*",
                                    "#001:004:01:064*",
                                    "#001:004:01:128*",
                                    "#001:004:01:064*",
                                    "#001:004:01:032*",
                                    "#001:004:01:016*",
                                    "#001:004:01:008*",
                                    "#001:004:01:004*",
                                    "#001:004:01:002*",
                                    "#001:004:01:001*"};
   private SerialPort serialPort;
   private String sResponse;
   
   public RunningLightsTask(SerialPort serialConn)
   {
       Running = false;
       iCurrentCommand = 0;
       serialPort = serialConn;
   }
   
   @Override
   public void run()
   {
      if (serialPort.isOpened() == true)
      {
        try 
        {
            //Send current command from string array out on to the RS485 bus
            serialPort.writeString(arsCommands[iCurrentCommand]);
            try 
            {
                //Get the response from the PiXtend eIO module or wait for timeout
                sResponse = serialPort.readString(arsCommands[iCurrentCommand].length(), 1000);
            } 
            catch (SerialPortTimeoutException ex) 
            {
                Logger.getLogger(FXMLController.class.getName()).log(Level.SEVERE, null, ex);
                sResponse = "Error!";
            }
        } 
        catch (SerialPortException ex) 
        {
           System.out.println(ex);
           sResponse = "Error!";
        }            
        //Print response to console / System out
        //System.out.println(sResponse);
        
        iCurrentCommand = iCurrentCommand + 1; 
        if (iCurrentCommand >= 16)
        {
            iCurrentCommand = 1;
        }
      }
      else
      {
          //The serialport is not open - print error message to output
          System.out.println("Error - Serial Port not open!!");
      }
   }
}
