package de.qubesolutions.pixtendeiodemosimple;

// This file is part of the PiXtend(R) Project.
//
// For more information about PiXtend(R) and this program,
// see <https://www.pixtend.de> or <https://www.pixtend.com>
//
// Copyright (C) 2019 Robin Turner
// Qube Solutions GmbH, Arbachtalstr. 6
// 72800 Eningen, Germany
//
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

import java.net.URL;
import java.util.ResourceBundle;
import java.util.logging.Level;
import java.util.logging.Logger;
import javafx.event.ActionEvent;
import javafx.fxml.FXML;
import javafx.fxml.Initializable;
import javafx.scene.control.Button;
import javafx.scene.control.ComboBox;
import javafx.scene.control.MenuItem;
import javafx.scene.control.TextField;
import javafx.scene.input.KeyEvent;

import jssc.SerialPort;
import jssc.SerialPortException;
import jssc.SerialPortList;
import jssc.SerialPortTimeoutException;

public class FXMLController implements Initializable {

    @FXML
    private Button btnSend;
    @FXML
    private ComboBox<String> cbCommand;
    @FXML
    private MenuItem mnuClose;
    @FXML
    private TextField txtAddress;
    @FXML
    private TextField txtIoNumber;
    @FXML
    private TextField txtValue;
    @FXML
    private TextField txtResponse;
    @FXML
    private TextField txtCommand;
    
    @Override
    public void initialize(URL url, ResourceBundle rb) {
        cbCommand.getItems().addAll("Read Digital Input","Write Digital Output",
                "Read Analog Input","Write Analog Output","Read Status Digital", "Read Status Analog");
        cbCommand.setValue("Read Digital Input");
    }    

    @FXML
    private void handlebtnSendAction(ActionEvent event) {
        String sResponse;
        String sTest;
        sTest = BuildProtocolMessage();
        
        txtCommand.setText(sTest);
        
        //Send PiXtend eIO Protocol/Message out onto the bus
 
        // initialization with port
        SerialPort serialPort = new SerialPort("COM3");
 
        try {
            // open port for communication
            serialPort.openPort();
            // baundRate 19200, numberOfDataBits 8, numberOfStopBits 1, parity EVEN
            serialPort.setParams(19200, 8, 1, 2);
            // String data transfer
            serialPort.writeString(sTest);
            try {
                //Wait
                Thread.sleep(100);
            } catch (InterruptedException ex) {
                Logger.getLogger(FXMLController.class.getName()).log(Level.SEVERE, null, ex);
            }
            try {
                sResponse = serialPort.readString(sTest.length(), 1000);
                txtResponse.setText(sResponse);
            } catch (SerialPortTimeoutException ex) {
                txtResponse.setText("Error no response!");
                Logger.getLogger(FXMLController.class.getName()).log(Level.SEVERE, null, ex);
            }
           // close port
           serialPort.closePort();  
        } catch (SerialPortException ex) {
           System.out.println(ex);
        }        
    }

    @FXML
    private void handlecbCommandAction(ActionEvent event) {
        String sTest;
        sTest = BuildProtocolMessage();
        txtCommand.setText(sTest);     
    }

    @FXML
    private void handletxtAddressOnKeyTyped(KeyEvent event) {
        String sTest;
        sTest = BuildProtocolMessage();
        txtCommand.setText(sTest);      
    }

    @FXML
    private void handletxtIoNumberOnKeyTyped(KeyEvent event) {
        String sTest;
        sTest = BuildProtocolMessage();
        txtCommand.setText(sTest);        
    }

    @FXML
    private void handletxtValueOnKeyTyped(KeyEvent event) {
        String sTest;
        sTest = BuildProtocolMessage();
        txtCommand.setText(sTest);        
    }
    
    private String BuildProtocolMessage()
    {
        //Declarations
        Integer iModuleType = 0;
        String sTemp;
        
        //Start of PiXtend eIO Protocol
        sTemp = "#";
        
        //Check length of device Address and pre-pend "0" (Zeros)
        if (txtAddress.getText().length() == 0)
        {
            sTemp = sTemp.concat("000");
        }        
        if (txtAddress.getText().length() == 1)
        {
            sTemp = sTemp.concat("00");
        }
        if (txtAddress.getText().length() == 2)
        {
            sTemp = sTemp.concat("0");
        }   
        
        //Concat the device Address to the protocol/message
        sTemp = sTemp.concat(txtAddress.getText());
        
        //Add divider
        sTemp = sTemp.concat(":");
        
        //Check for null to see if combobox is empty
        if (null == cbCommand.getValue())
        {
            sTemp = sTemp.concat("000");
        }
        else
        {
            //Add the chosen command to the protocol/message
            switch (cbCommand.getValue()) 
            {
                case "Read Digital Input":
                    sTemp = sTemp.concat("001");
                    iModuleType = 1;
                    break;
                case "Write Digital Output":
                    sTemp = sTemp.concat("003");
                    iModuleType = 1;
                    break;
                case "Read Analog Input":
                    sTemp = sTemp.concat("005");
                    iModuleType = 2;
                    break;
                case "Write Analog Output":
                    sTemp = sTemp.concat("006");
                    iModuleType = 2;
                    break;
                case "Read Status Digital":
                    sTemp = sTemp.concat("200");
                    iModuleType = 1;
                    break;
                case "Read Status Analog":
                    sTemp = sTemp.concat("200");
                    iModuleType = 2;
                    break;
                default:
                    sTemp = sTemp.concat("000");
                    iModuleType = 1;
                    break;
            }
        }

        //Add divider
        sTemp = sTemp.concat(":");
        
        //Determine length of IO number number and pre-pend a zero if needed
        if (txtIoNumber.getText().length() == 0)
        {
            sTemp = sTemp.concat("00");
        }        
        if (txtIoNumber.getText().length() == 1)
        {
            sTemp = sTemp.concat("0");
        }
        //Add IO number to protocol/message
        sTemp = sTemp.concat(txtIoNumber.getText());
        
        //Add divider
        sTemp = sTemp.concat(":");
        
        //Check string length of value and pre-pend zeros (0)
        switch (txtValue.getText().length()) {
            case 0:
                if (iModuleType == 1)
                {
                    sTemp = sTemp.concat("000");
                }   
                if (iModuleType == 2)
                {
                    sTemp = sTemp.concat("0000");
                }   break;
            case 1:
                if (iModuleType == 1)
                {
                    sTemp = sTemp.concat("00");
                }   
                if (iModuleType == 2)
                {
                    sTemp = sTemp.concat("000");
                }   break;
            case 2:
                if (iModuleType == 1)
                {
                    sTemp = sTemp.concat("0");
                }   
                if (iModuleType == 2)
                {
                    sTemp = sTemp.concat("00");
                }   break;
            case 3:
                if (iModuleType == 2)
                {
                    sTemp = sTemp.concat("0");
                }   break;
            default:
                break;                   
        }
        
        //Add value to the protocol/message
        sTemp = sTemp.concat(txtValue.getText());
        
        //Add End-Signal / End-Maker to protocol/message
        sTemp = sTemp.concat("*");
        
        return sTemp;
    }

    @FXML
    private void handlemnuClose(ActionEvent event) {
        //Exit program
        System.exit(0);
    }
}